""" from https://github.com/keithito/tacotron """
import re
from .u2a import u2a
from .numbers_normalization import normalize_numbers

"""
Cleaners are transformations that run over the input text at both training and
eval time.

Cleaners can be selected by passing a comma-delimited list of cleaner names as
the "cleaners" hyperparameter.
Some cleaners are English-specific. You'll typically want to use:
1. "english_cleaners" for English text
2. "transliteration_cleaners" for non-English text that can be transliterated
to ASCII using the Unidecode library (https://pypi.python.org/pypi/Unidecode)
3. "basic_cleaners" if you do not want to transliterate (in this case, you
should also update the symbols in symbols.py to match your data).
"""


# Regular expression matching whitespace:
_whitespace_re = re.compile(r'\s+')

# List of (regular expression, replacement) pairs for abbreviations:
_abbreviations = [(re.compile('\\b%s\\.' % x[0], re.IGNORECASE), x[1])
                  for x in [
                            ('mrs', 'misess'),
                            ('mr', 'mister'),
                            ('dr', 'doctor'),
                            ('st', 'saint'),
                            ('co', 'company'),
                            ('jr', 'junior'),
                            ('maj', 'major'),
                            ('gen', 'general'),
                            ('drs', 'doctors'),
                            ('rev', 'reverend'),
                            ('lt', 'lieutenant'),
                            ('hon', 'honorable'),
                            ('sgt', 'sergeant'),
                            ('capt', 'captain'),
                            ('esq', 'esquire'),
                            ('ltd', 'limited'),
                            ('col', 'colonel'),
                            ('ft', 'fort'),
]]


class Cleaner:
    def __init__(self, convert_to_ascii=None,
                 cleaner_names=["english_cleaners"]):
        if convert_to_ascii is not None:
            self.convert_to_ascii = convert_to_ascii
        self.cleaner_names = cleaner_names

    @staticmethod
    def expand_abbreviations(text):
        for regex, replacement in _abbreviations:
            text = re.sub(regex, replacement, text)
        return text

    @staticmethod
    def expand_numbers(text):
        return normalize_numbers(text)

    @staticmethod
    def lowercase(text):
        return text.lower()

    @staticmethod
    def collapse_whitespace(text):
        return re.sub(_whitespace_re, ' ', text)

    @staticmethod
    def convert_to_ascii(text):
        return u2a(text)

    def basic_cleaners(self, text):
        """
        Basic pipeline that lowercases and collapses whitespace without
        transliteration.
        """
        text = self.lowercase(text)
        text = self.collapse_whitespace(text)
        return text

    def transliteration_cleaners(self, text):
        """Pipeline for non-English text that transliterates to ASCII."""
        text = self.convert_to_ascii(text)
        text = self.lowercase(text)
        text = self.collapse_whitespace(text)
        return text

    def english_cleaners(self, text):
        """
        Pipeline for English text, including number and abbreviation
        expansion.
        """
        text = self.convert_to_ascii(text)
        text = self.lowercase(text)
        text = self.expand_numbers(text)
        text = self.expand_abbreviations(text)
        text = self.collapse_whitespace(text)
        return text

    def clean(self, text, cleaner_names=None):
        if cleaner_names is None:
            cleaner_names = self.cleaner_names

        for name in cleaner_names:
            cleaner = getattr(self, name)
            if not cleaner:
                raise Exception('Unknown cleaner: %s' % name)
            text = cleaner(text)
        return text



""" from https://github.com/keithito/tacotron """

from .cleaners import Cleaner
from .cmudict import CMUDict

import re
from .symbols import symbols

# Mappings from symbol to numeric ID and vice versa:
_symbol_to_id = {s: i for i, s in enumerate(symbols)}
_id_to_symbol = {i: s for i, s in enumerate(symbols)}

# Regular expression matching text enclosed in curly braces:
_curly_re = re.compile(r'(.*?)\{(.+?)\}(.*)')


class Text2Id:
    def __init__(self, cmu_dict_path=None, convert_to_ascii=None):
        """
        Class for preprocessing text before feeding to text to speech neural
        network.
        :param cmu_dict_path: path to CMU Pronouncing Dictionary, if None load
        default.
        :param convert_to_ascii: function or callable object for converting
        unicode text ot ascii.
        """
        self.cmudict = CMUDict(cmu_dict_path)
        self.cleaner = Cleaner(convert_to_ascii)

    def get_arpabet(self, word):
        word_arpabet = self.cmudict.lookup(word)
        if word_arpabet is not None:
            return "{" + word_arpabet[0] + "}"
        else:
            return word

    def text_to_sequence(self, text, cleaner_names=None, use_dict=True):
        """
        Converts a string of text to a sequence of IDs corresponding to the
        symbols in the text.

          The text can optionally have ARPAbet sequences enclosed in curly
          braces embedded in it. For example,
          "Turn left on {HH AW1 S S T AH0 N} Street."

          Args:
            text: string to convert to a sequence
            cleaner_names: names of the cleaner functions to run the text
            throught
            use_dict: if true then converts symbols to arpabet
            (https://en.wikipedia.org/wiki/ARPABET)

          Returns:
            List of integers corresponding to the symbols in the text
        """
        sequence = []

        space = self._symbols_to_sequence(' ')
        # Check for curly braces and treat their contents as ARPAbet:
        while len(text):
            m = _curly_re.match(text)
            if not m:
                clean_text = self.cleaner.clean(text, cleaner_names)
                if use_dict:
                    clean_text = [self.get_arpabet(w)
                                  for w in clean_text.split(" ")]
                    for i in range(len(clean_text)):
                        t = clean_text[i]
                        if t.startswith("{"):
                            sequence += self._arpabet_to_sequence(t[1:-1])
                        else:
                            sequence += self._symbols_to_sequence(t)
                        sequence += space
                else:
                    sequence += self._symbols_to_sequence(clean_text)
                break
            sequence += self._symbols_to_sequence(
                self.cleaner.clean(m.group(1), cleaner_names))
            sequence += self._arpabet_to_sequence(m.group(2))
            text = m.group(3)

        # remove trailing space
        if use_dict:
            sequence = sequence[:-1] if sequence[-1] == space[0] else sequence
        return sequence

    def sequence_to_text(self, sequence):
        """
        Converts a sequence of IDs back to a string
        """
        result = ''
        for symbol_id in sequence:
            if symbol_id in _id_to_symbol:
                s = _id_to_symbol[symbol_id]
                # Enclose ARPAbet back in curly braces:
                if len(s) > 1 and s[0] == '@':
                    s = '{%s}' % s[1:]
                result += s
        return result.replace('}{', ' ')

    def _symbols_to_sequence(self, symbols):
        return [_symbol_to_id[s] for s in symbols
                if self._should_keep_symbol(s)]

    def _arpabet_to_sequence(self, text):
        return self._symbols_to_sequence(['@' + s for s in text.split()])

    @staticmethod
    def _should_keep_symbol(s):
        return s in _symbol_to_id and s != '_' and s != '~'


if __name__ == "__main__":
    t2i = Text2Id()
    text = "Müller said 11 12 13"
    print(t2i.text_to_sequence(text))
    print(t2i.sequence_to_text(t2i.text_to_sequence(text)))
    print(t2i.text_to_sequence(text, use_dict=False))
    print(t2i.sequence_to_text(t2i.text_to_sequence(text, use_dict=False)))



""" from https://github.com/keithito/tacotron """
from .cmudict import valid_symbols

"""
Defines the set of symbols used in text input to the model.
The default is a set of ASCII characters that works well for English or text
that has been run through Unidecode. For other data, you can modify
_characters. See TRAINING_DATA.md for details.
"""

_pad = '_'
_punctuation = '!\'(),.:;? '
_special = '-'
_letters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'

"""
Prepend "@" to ARPAbet symbols to ensure uniqueness
(some are the same as uppercase letters):
"""
_arpabet = ['@' + s for s in valid_symbols]

# Export all symbols:
symbols = [_pad] + list(_special) + list(_punctuation) + \
          list(_letters) + _arpabet



from setuptools import find_packages, setup

readme = open('README.md').read()

VERSION = '0.0.0'

requirements = [
    'inflect',
]

setup(
    # Metadata
    name='text2id',
    version=VERSION,
    author='Andrey Anufriev',
    author_email='a.a.egorovich@gmail.com',
    url='https://github.com/andreyanuf/text2id',
    description='Text converter for text to speech tasks.',
    long_description=readme,
    long_description_content_type='text/markdown',
    license='MIT',
    packages=find_packages(),
    package_data={'': ['text2id/data/cmudict']},
    include_package_data=True,
    zip_safe=False,
    install_requires=requirements,
    # Classifiers
    classifiers=[
        'Programming Language :: Python :: 3',
    ],
)